# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['asynckivy']

package_data = \
{'': ['*']}

install_requires = \
['asyncgui>=0.2.1,<0.3.0']

setup_kwargs = {
    'name': 'asynckivy',
    'version': '0.2.1',
    'description': 'Async library for Kivy',
    'long_description': '# AsyncKivy\n\n[Youtube](https://www.youtube.com/playlist?list=PLNdhqAjzeEGjTpmvNck4Uykps8s9LmRTJ)  \n[日本語doc](README_jp.md)  \n\n## Installation\n\n```\n# stable version\npip install asynckivy\n```\n\n```\n# development version\npip install git+https://github.com/gottadiveintopython/asynckivy.git@master#egg=asynckivy\n```\n\n## Usage\n\n```python\nimport asynckivy as ak\n\nasync def some_task(button):\n    # wait for 1sec\n    dt = await ak.sleep(1)\n    print(f\'{dt} seconds have passed\')\n    \n    # wait until a button is pressed\n    await ak.event(button, \'on_press\')\n\n    # wait until \'button.x\' changes\n    __, x = await ak.event(button, \'x\')\n    print(f\'button.x is now {x}\')\n\n    # wait until \'button.x\' becomes greater than 100\n    if button.x <= 100:\n        __, x = await ak.event(button, \'x\', filter=lambda __, x: x>100)\n        print(f\'button.x is now {x}\')\n\n    # create a new thread, run a function on it, then\n    # wait for the completion of that thread\n    r = await ak.run_in_thread(some_heavy_task)\n    print(f"result of \'some_heavy_task()\': {r}")\n\n    # wait until EITHER a button is pressed OR 5sec passes\n    tasks = await ak.or_(\n        ak.event(button, \'on_press\'),\n        ak.sleep(5),\n    )\n    print("The button was pressed" if tasks[0].done else "5sec passed")\n\n    # wait until BOTH a button is pressed AND 5sec passes"\n    tasks = await ak.and_(\n        ak.event(button, \'on_press\'),\n        ak.sleep(5),\n    )\n\nak.start(some_task(some_button))\n```\n\n### animation\n\n```python\nimport asynckivy as ak\n\nasync def some_task(widget):\n    # start an animation and wait for its completion\n    await ak.animate(widget, width=200, t=\'in_out_quad\', d=.5)\n\n    # Interpolate between the values 0 and 200.\n    async for v in ak.interpolate(0, 200, s=.2, d=2, t=\'linear\'):\n        print(v)\n        # await ak.sleep(1)  # Do not await anything during the iteration\n\n    # change the text of Label with fade-transition\n    label = Label(...)\n    async with ak.fade_transition(label):\n        label.text = \'new text\'\n```\n\n### touch handling\n\nYou can easily handle `on_touch_xxx` events via `asynckivy.rest_of_touch_moves()`.\n\n```python\nimport asynckivy as ak\n\nclass Painter(RelativeLayout):\n    def on_touch_down(self, touch):\n        if self.collide_point(*touch.opos):\n            ak.start(self.draw_rect(touch))\n            return True\n    \n    async def draw_rect(self, touch):\n        from kivy.graphics import Line, Color, Rectangle\n        from kivy.utils import get_random_color\n        with self.canvas:\n            Color(*get_random_color())\n            line = Line(width=2)\n        ox, oy = self.to_local(*touch.opos)\n        async for __ in ak.rest_of_touch_moves(self, touch):\n            # This part is iterated everytime \'on_touch_move\' is fired.\n            x, y = self.to_local(*touch.pos)\n            min_x = min(x, ox)\n            min_y = min(y, oy)\n            max_x = max(x, ox)\n            max_y = max(y, oy)\n            line.rectangle = [min_x, min_y, max_x - min_x, max_y - min_y]\n            # await ak.sleep(1)  # Do not await anything during the iteration\n\n        # If you want to do something when \'on_touch_up\' is fired, do it here.\n        do_something_on_touch_up()\n```\n\n### synchronization primitive\n\nThere is a Trio\'s [Event](https://trio.readthedocs.io/en/stable/reference-core.html#trio.Event) equivalent.\n\n```python\nimport asynckivy as ak\n\nasync def task_A(e):\n    print(\'A1\')\n    await e.wait()\n    print(\'A2\')\nasync def task_B(e):\n    print(\'B1\')\n    await e.wait()\n    print(\'B2\')\n\ne = ak.Event()\nak.start(task_A(e))\n# A1\nak.start(task_B(e))\n# B1\ne.set()\n# A2\n# B2\n```\n\n### Misc\n\n```python\nimport asynckivy as ak\n\n# schedule a coroutine/Task to start after the next frame\nak.start_soon(coro_or_task)\n```\n\n## planned api break\n\n- remove `animation()`, the older name of `animate()`\n- remove `all_touch_moves()`, the older name of `rest_of_touch_moves()`\n- remove `process_and_thread.thread()`, the older name of `run_in_thread()`\n\n## Test Environment\n\n- CPython 3.7 + Kivy 1.11.1\n- CPython 3.7 + Kivy 2.0.0\n- CPython 3.8 + Kivy 2.0.0\n- CPython 3.9 + Kivy 2.0.0\n\n## Why this does exist\n\nKivy supports two legit async libraries, [asyncio][asyncio] and [Trio][trio], from version 2.0.0 so developing another one seems [reinventing the wheel][reinventing]. Actually, I started developing this library just for learning how async/await works so it *was* initially `reinventing the wheel`.\n\nBut after playing with Trio and Kivy for a while, I noticed that Trio is not suitable for the situation where fast reactions are required e.g. touch events. The same is true of asyncio. You can see why by running `examples/misc/why_xxx_is_not_suitable_for_handling_touch_events.py`, and masshing a mouse button. You\'ll see sometimes the printed `up` and `down` aren\'t paired. You\'ll see the printed coordinates aren\'t relative to the `RelativeLayout` even though the `target` belongs to it.\n\nThe cause of those problems is that calling `trio.Event.set()` / `asyncio.Event.set()` doesn\'t *immediately* resume the tasks that are waiting for the `Event` to be set. It just schedules the tasks to resume.\nSame thing can be said to `nursery.start_soon()` and `asyncio.create_task()`. Yes, Trio has `nursery.start()`, which immediately starts a task, but it\'s an async-function so it cannot be called from synchronous code, which means it\'s no use here.\n\nTrio and asyncio are async **I/O** libraries after all. They probably don\'t need the functionality that immediately resumes/starts tasks, which is necessary for Kivy\'s touch handling.\nThier core design may not be suitable for GUI in the first place.\nThat\'s why I\'m still developing this `asynckivy` library to this day.\n\n[asyncio]:https://docs.python.org/3/library/asyncio.html\n[trio]:https://trio.readthedocs.io/en/stable/\n[reinventing]:https://en.wikipedia.org/wiki/Reinventing_the_wheel\n',
    'author': 'Nattōsai Mitō',
    'author_email': 'flow4re2c@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/gottadiveintopython/asynckivy',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
